@@ -1,5 +1,6 @@
 /**
- *  Copyright 2014 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -13,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package brut.apktool;
 
 import brut.androlib.*;
@@ -74,7 +74,6 @@ public class Main {
             setAdvanceMode(true);
         }
 
-        // @todo use new ability of apache-commons-cli to check hasOption for non-prefixed items
         boolean cmdFound = false;
         for (String opt : commandLine.getArgs()) {
             if (opt.equalsIgnoreCase("d") || opt.equalsIgnoreCase("decode")) {
@@ -96,9 +95,10 @@ public class Main {
         }
 
         // if no commands ran, run the version / usage check.
-        if (cmdFound == false) {
+        if (!cmdFound) {
             if (commandLine.hasOption("version")) {
                 _version();
+                System.exit(0);
             } else {
                 usage();
             }
@@ -109,7 +109,7 @@ public class Main {
         ApkDecoder decoder = new ApkDecoder();
 
         int paraCount = cli.getArgList().size();
-        String apkName = (String) cli.getArgList().get(paraCount - 1);
+        String apkName = cli.getArgList().get(paraCount - 1);
         File outDir;
 
         // check for options
@@ -187,14 +187,17 @@ public class Main {
         } catch (DirectoryException ex) {
             System.err.println("Could not modify internal dex files. Please ensure you have permission.");
             System.exit(1);
+        } finally {
+            try {
+                decoder.close();
+            } catch (IOException ignored) {}
         }
-
     }
 
     private static void cmdBuild(CommandLine cli) throws BrutException {
         String[] args = cli.getArgs();
         String appDirName = args.length < 2 ? "." : args[1];
-        File outFile = null;
+        File outFile;
         ApkOptions apkOptions = new ApkOptions();
 
         // check for build options
@@ -229,7 +232,7 @@ public class Main {
 
     private static void cmdInstallFramework(CommandLine cli) throws AndrolibException {
         int paraCount = cli.getArgList().size();
-        String apkName = (String) cli.getArgList().get(paraCount - 1);
+        String apkName = cli.getArgList().get(paraCount - 1);
 
         ApkOptions apkOptions = new ApkOptions();
         if (cli.hasOption("p") || cli.hasOption("frame-path")) {
@@ -243,7 +246,7 @@ public class Main {
 
     private static void cmdPublicizeResources(CommandLine cli) throws AndrolibException {
         int paraCount = cli.getArgList().size();
-        String apkName = (String) cli.getArgList().get(paraCount - 1);
+        String apkName = cli.getArgList().get(paraCount - 1);
 
         new Androlib().publicizeResources(new File(apkName));
     }
@@ -444,8 +447,6 @@ public class Main {
     }
 
     private static void usage() {
-
-        // load basicOptions
         _Options();
         HelpFormatter formatter = new HelpFormatter();
         formatter.setWidth(120);
@@ -482,7 +483,7 @@ public class Main {
                         + "For smali/baksmali info, see: https://github.com/JesusFreke/smali");
     }
 
-    private static void setupLogging(Verbosity verbosity) {
+    private static void setupLogging(final Verbosity verbosity) {
         Logger logger = Logger.getLogger("");
         for (Handler handler : logger.getHandlers()) {
             logger.removeHandler(handler);
@@ -505,7 +506,13 @@ public class Main {
                     if (record.getLevel().intValue() >= Level.WARNING.intValue()) {
                         System.err.write(message.getBytes());
                     } else {
-                        System.out.write(message.getBytes());
+                        if (record.getLevel().intValue() >= Level.INFO.intValue()) {
+                            System.out.write(message.getBytes());
+                        } else {
+                            if (verbosity == Verbosity.VERBOSE) {
+                                System.out.write(message.getBytes());
+                            }
+                        }
                     }
                 } catch (Exception exception) {
                     reportError(null, exception, ErrorManager.FORMAT_FAILURE);
@@ -542,8 +549,8 @@ public class Main {
         Main.advanceMode = advanceMode;
     }
 
-    private static enum Verbosity {
-        NORMAL, VERBOSE, QUIET;
+    private enum Verbosity {
+        NORMAL, VERBOSE, QUIET
     }
 
     private static boolean advanceMode = false;
