@@ -1,5 +1,6 @@
 /**
- *  Copyright 2014 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -13,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package brut.androlib.res.data.value;
 
 import brut.androlib.AndrolibException;
@@ -23,6 +23,7 @@ import brut.androlib.res.xml.ResValuesXmlSerializable;
 import brut.util.Duo;
 import java.io.IOException;
 import org.xmlpull.v1.XmlSerializer;
+import java.util.logging.Logger;
 
 /**
  * @author Ryszard Wiśniewski <brut.alll@gmail.com>
@@ -52,6 +53,13 @@ public class ResStyleValue extends ResBagValue implements
         }
         for (int i = 0; i < mItems.length; i++) {
             ResResSpec spec = mItems[i].m1.getReferent();
+
+            if (spec == null) {
+                LOGGER.fine(String.format("null reference: m1=0x%08x(%s), m2=0x%08x(%s)",
+                        mItems[i].m1.getRawIntValue(), mItems[i].m1.getType(), mItems[i].m2.getRawIntValue(), mItems[i].m2.getType()));
+                continue;
+            }
+
             String name = null;
             String value = null;
 
@@ -85,4 +93,6 @@ public class ResStyleValue extends ResBagValue implements
     }
 
     private final Duo<ResReferenceValue, ResScalarValue>[] mItems;
+
+    private static final Logger LOGGER = Logger.getLogger(ResStyleValue.class.getName());
 }
