@@ -1,5 +1,6 @@
 /**
- *  Copyright 2014 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Ryszard Wiśniewski <brut.alll@gmail.com>
+ *  Copyright (C) 2017 Connor Tumbleson <connor.tumbleson@gmail.com>
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -13,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package brut.androlib.res.decoder;
 
 import brut.androlib.res.xml.ResXmlEncoders;
@@ -131,8 +131,14 @@ public class StringBlock {
         if (style == null) {
             return ResXmlEncoders.escapeXmlChars(raw);
         }
+
+        // If the returned style is further in string, than string length. Lets skip it.
+        if (style[1] > raw.length()) {
+            return ResXmlEncoders.escapeXmlChars(raw);
+        }
         StringBuilder html = new StringBuilder(raw.length() + 32);
         int[] opened = new int[style.length / 3];
+        boolean[] unclosed = new boolean[style.length / 3];
         int offset = 0, depth = 0;
         while (true) {
             int i = -1, j;
@@ -149,6 +155,9 @@ public class StringBlock {
                 int last = opened[j];
                 int end = style[last + 2];
                 if (end >= start) {
+                    if (style[last + 1] == -1 && end != -1) {
+                        unclosed[j] = true;
+                    }
                     break;
                 }
                 if (offset <= end) {
@@ -160,6 +169,11 @@ public class StringBlock {
             depth = j + 1;
             if (offset < start) {
                 html.append(ResXmlEncoders.escapeXmlChars(raw.substring(offset, start)));
+                if (j >= 0 && unclosed.length >= j && unclosed[j]) {
+                    if (unclosed.length > (j + 1) && unclosed[j + 1] || unclosed.length == 1) {
+                        outputStyleTag(getString(style[opened[j]]), html, true);
+                    }
+                }
                 offset = start;
             }
             if (i == -1) {
@@ -303,32 +317,34 @@ public class StringBlock {
     private static final int[] getUtf8(byte[] array, int offset) {
         int val = array[offset];
         int length;
-
+        // We skip the utf16 length of the string
         if ((val & 0x80) != 0) {
             offset += 2;
         } else {
-            offset += 1;
+            offset += 1;	
         }
+        // And we read only the utf-8 encoded length of the string
         val = array[offset];
+        offset += 1;
         if ((val & 0x80) != 0) {
-            offset += 2;
-        } else {
+        	int low = (array[offset] & 0xFF); 
+        	length = ((val & 0x7F) << 8) + low;
             offset += 1;
-        }
-        length = 0;
-        while (array[offset + length] != 0) {
-            length++;
+        } else {
+            length = val;
         }
         return new int[] { offset, length};
     }
-
+    
     private static final int[] getUtf16(byte[] array, int offset) {
         int val = ((array[offset + 1] & 0xFF) << 8 | array[offset] & 0xFF);
 
-        if (val == 0x8000) {
+        if ((val & 0x8000) != 0) {
             int high = (array[offset + 3] & 0xFF) << 8;
             int low = (array[offset + 2] & 0xFF);
-            return new int[] {4, (high + low) * 2};
+            int len_value =  ((val & 0x7FFF) << 16) + (high + low);
+            return new int[] {4, len_value * 2};
+            
         }
         return new int[] {2, val * 2};
     }
